<?php

namespace Litepie\Event\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Event\Http\Requests\BusinessDirectoryRequest;
use Litepie\Event\Interfaces\BusinessDirectoryRepositoryInterface;
use Litepie\Event\Models\BusinessDirectory;

/**
 * Resource controller class for business_directory.
 */
class BusinessDirectoryResourceController extends BaseController
{

    /**
     * Initialize business_directory resource controller.
     *
     * @param type BusinessDirectoryRepositoryInterface $business_directory
     *
     * @return null
     */
    public function __construct(BusinessDirectoryRepositoryInterface $business_directory)
    {
        parent::__construct();
        $this->repository = $business_directory;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Event\Repositories\Criteria\BusinessDirectoryResourceCriteria::class);
    }

    /**
     * Display a list of business_directory.
     *
     * @return Response
     */
    public function index(BusinessDirectoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Event\Repositories\Presenter\BusinessDirectoryPresenter::class)
                ->$function();
        }

        $business_directories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('event::business_directory.names'))
            ->view('event::business_directory.index', true)
            ->data(compact('business_directories', 'view'))
            ->output();
    }

    /**
     * Display business_directory.
     *
     * @param Request $request
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function show(BusinessDirectoryRequest $request, BusinessDirectory $business_directory)
    {

        if ($business_directory->exists) {
            $view = 'event::business_directory.show';
        } else {
            $view = 'event::business_directory.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('event::business_directory.name'))
            ->data(compact('business_directory'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new business_directory.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(BusinessDirectoryRequest $request)
    {

        $business_directory = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('event::business_directory.name')) 
            ->view('event::business_directory.create', true) 
            ->data(compact('business_directory'))
            ->output();
    }

    /**
     * Create new business_directory.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(BusinessDirectoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $business_directory                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('event::business_directory.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/business_directory/' . $business_directory->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/event/business_directory'))
                ->redirect();
        }

    }

    /**
     * Show business_directory for editing.
     *
     * @param Request $request
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function edit(BusinessDirectoryRequest $request, BusinessDirectory $business_directory)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('event::business_directory.name'))
            ->view('event::business_directory.edit', true)
            ->data(compact('business_directory'))
            ->output();
    }

    /**
     * Update the business_directory.
     *
     * @param Request $request
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function update(BusinessDirectoryRequest $request, BusinessDirectory $business_directory)
    {
        try {
            $attributes = $request->all();

            $business_directory->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('event::business_directory.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/business_directory/' . $business_directory->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/business_directory/' . $business_directory->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the business_directory.
     *
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function destroy(BusinessDirectoryRequest $request, BusinessDirectory $business_directory)
    {
        try {

            $business_directory->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::business_directory.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('event/business_directory/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/business_directory/' . $business_directory->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple business_directory.
     *
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function delete(BusinessDirectoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::business_directory.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('event/business_directory'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/business_directory'))
                ->redirect();
        }

    }

    /**
     * Restore deleted business_directories.
     *
     * @param Model   $business_directory
     *
     * @return Response
     */
    public function restore(BusinessDirectoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('event::business_directory.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/event/business_directory'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/business_directory/'))
                ->redirect();
        }

    }

}
