<?php

namespace Litepie\Event\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Event\Http\Requests\EventRequest;
use Litepie\Event\Interfaces\EventRepositoryInterface;
use Litepie\Event\Models\Event;

/**
 * Resource controller class for event.
 */
class EventResourceController extends BaseController
{

    /**
     * Initialize event resource controller.
     *
     * @param type EventRepositoryInterface $event
     *
     * @return null
     */
    public function __construct(EventRepositoryInterface $event)
    {
        parent::__construct();
        $this->repository = $event;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Event\Repositories\Criteria\EventResourceCriteria::class);
    }

    /**
     * Display a list of event.
     *
     * @return Response
     */
    public function index(EventRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Event\Repositories\Presenter\EventPresenter::class)
                ->$function();
        }

        $events = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('event::event.names'))
            ->view('event::event.index', true)
            ->data(compact('events', 'view'))
            ->output();
    }

    /**
     * Display event.
     *
     * @param Request $request
     * @param Model   $event
     *
     * @return Response
     */
    public function show(EventRequest $request, Event $event)
    {

        if ($event->exists) {
            $view = 'event::event.show';
        } else {
            $view = 'event::event.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('event::event.name'))
            ->data(compact('event'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new event.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(EventRequest $request)
    {

        $event = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('event::event.name')) 
            ->view('event::event.create', true) 
            ->data(compact('event'))
            ->output();
    }

    /**
     * Create new event.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(EventRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $event                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('event::event.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/event/' . $event->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/event/event'))
                ->redirect();
        }

    }

    /**
     * Show event for editing.
     *
     * @param Request $request
     * @param Model   $event
     *
     * @return Response
     */
    public function edit(EventRequest $request, Event $event)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('event::event.name'))
            ->view('event::event.edit', true)
            ->data(compact('event'))
            ->output();
    }

    /**
     * Update the event.
     *
     * @param Request $request
     * @param Model   $event
     *
     * @return Response
     */
    public function update(EventRequest $request, Event $event)
    {
        try {
            $attributes = $request->all();

            $event->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('event::event.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/event/' . $event->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/event/' . $event->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the event.
     *
     * @param Model   $event
     *
     * @return Response
     */
    public function destroy(EventRequest $request, Event $event)
    {
        try {

            $event->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::event.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('event/event/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/event/' . $event->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple event.
     *
     * @param Model   $event
     *
     * @return Response
     */
    public function delete(EventRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::event.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('event/event'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/event'))
                ->redirect();
        }

    }

    /**
     * Restore deleted events.
     *
     * @param Model   $event
     *
     * @return Response
     */
    public function restore(EventRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('event::event.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/event/event'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/event/'))
                ->redirect();
        }

    }

}
