<?php

namespace Litepie\Event\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Event\Http\Requests\ProgramRequest;
use Litepie\Event\Interfaces\ProgramRepositoryInterface;
use Litepie\Event\Models\Program;

/**
 * Resource controller class for program.
 */
class ProgramResourceController extends BaseController
{

    /**
     * Initialize program resource controller.
     *
     * @param type ProgramRepositoryInterface $program
     *
     * @return null
     */
    public function __construct(ProgramRepositoryInterface $program)
    {
        parent::__construct();
        $this->repository = $program;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Event\Repositories\Criteria\ProgramResourceCriteria::class);
    }

    /**
     * Display a list of program.
     *
     * @return Response
     */
    public function index(ProgramRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Event\Repositories\Presenter\ProgramPresenter::class)
                ->$function();
        }

        $programs = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('event::program.names'))
            ->view('event::program.index', true)
            ->data(compact('programs', 'view'))
            ->output();
    }

    /**
     * Display program.
     *
     * @param Request $request
     * @param Model   $program
     *
     * @return Response
     */
    public function show(ProgramRequest $request, Program $program)
    {

        if ($program->exists) {
            $view = 'event::program.show';
        } else {
            $view = 'event::program.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('event::program.name'))
            ->data(compact('program'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new program.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ProgramRequest $request)
    {

        $program = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('event::program.name')) 
            ->view('event::program.create', true) 
            ->data(compact('program'))
            ->output();
    }

    /**
     * Create new program.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ProgramRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $program                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('event::program.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/program/' . $program->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/event/program'))
                ->redirect();
        }

    }

    /**
     * Show program for editing.
     *
     * @param Request $request
     * @param Model   $program
     *
     * @return Response
     */
    public function edit(ProgramRequest $request, Program $program)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('event::program.name'))
            ->view('event::program.edit', true)
            ->data(compact('program'))
            ->output();
    }

    /**
     * Update the program.
     *
     * @param Request $request
     * @param Model   $program
     *
     * @return Response
     */
    public function update(ProgramRequest $request, Program $program)
    {
        try {
            $attributes = $request->all();

            $program->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('event::program.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('event/program/' . $program->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/program/' . $program->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the program.
     *
     * @param Model   $program
     *
     * @return Response
     */
    public function destroy(ProgramRequest $request, Program $program)
    {
        try {

            $program->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::program.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('event/program/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('event/program/' . $program->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple program.
     *
     * @param Model   $program
     *
     * @return Response
     */
    public function delete(ProgramRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('event::program.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('event/program'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/program'))
                ->redirect();
        }

    }

    /**
     * Restore deleted programs.
     *
     * @param Model   $program
     *
     * @return Response
     */
    public function restore(ProgramRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('event::program.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/event/program'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/event/program/'))
                ->redirect();
        }

    }

}
